@@ -43,6 +43,9 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef WIN32
+#include <io.h>
+#endif
 
 /* many places for getopt */
 #ifdef HAVE_GETOPT_H
@@ -59,6 +62,12 @@
 #define RAPTOR_DEBUG 2
 #endif
 
+#ifndef RAPTOR_INTERNAL
+#define RAPTOR_MALLOC(type, size)		(type)malloc(size)
+#define RAPTOR_CALLOC(type, nmemb, size)	(type)calloc(nmemb, size)
+#define RAPTOR_FREE(type, ptr)			free((void*)ptr)
+#endif
+
 #ifdef NEED_OPTIND_DECLARATION
 extern int optind;
 extern char *optarg;
@@ -576,7 +585,10 @@ rdfdiff_add_blank_statement_owner(rdfdiff_file* file,
                                (char*)statement->object->value.blank.string);
   if(!blank)
     goto failed;
-  
+
+  if(blank->owner)
+    raptor_free_statement(blank->owner);
+
   blank->owner = raptor_statement_copy(statement);
   if(!blank->owner)
     goto failed;
